/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* Peers facility.
 *
 * Handle the peers.
 *
 * See peer-api.h and peer.h for more information on the functions 
 *
 */

#include <pthread.h>
#include <errno.h>

#include "waaad-internal.h"

/* States of a peer */
typedef enum {
	/* Stable states */
	STATE_DISABLED = 1,	/* No connexion must be attempted */
	STATE_OPEN,		/* Connexion established */
	/* Transitional states */
	STATE_CLOSED,		/* No connexion established */
	STATE_CLOSING,		/* the connexion is being shutdown */
	STATE_WAIT_CNX,		/* Trying to establish a connexion at transport level, waiting for ACK/NACK or new cnx initiated by remote peer (concurrent attempts) */
	STATE_WAIT_CEA		/* CER has been sent, waiting for CEA or timeout */
	/* TBC... */

	/* Other states from the RFC: are they needed? Wait-Conn-Ack/Elect, Wait-Returns */
	/* Also need the states for "Suspect" peers from the watchdog mechanism. */
	
} peer_state_t;

/* List of sent requests, ordered by hop-by-hop id */
typedef struct _sent_req {
	/* Chaining information */
	struct _sent_r	*next;	/* The next sent request */
	struct _sent_r	*prev;	/* The previous sent request */

	/* Pointer to the message */
	msg_t		*msg;

	/* The following two fields are copied from message for faster processing on message reception */
	uint32_t	 hbh;	/* The hop-by-hop value of the message */
	int	 	 rtb;	/* Is this a routable message? */
	
} _sent_req_t;

/* Peer internal description */
typedef struct _peer {
	char 		*p_diamid;	/* The Diameter-Id of this peer */
	
	struct _peer	*p_next;	/* The next peer in the global list, sorted by Diameter-Id */
	struct _peer	*p_prev;	/* The previous peer in the global list, sorted by Diameter-Id */
	
	peer_state_t	 p_state;	/* State of the peer */

	struct _peer	*p_nextval;	/* The next peer in the STATE_OPEN state, or NULL if this peer is not in this state */
	
	pthread_t	*p_thread;	/* The thread handling this peer */

	uint32_t	 p_hbh;		/* current Hop-by-hop value */
	meq_t 		*p_outgoing;	/* Outgoing message queue */
	_sent_req_t	 p_sent;	/* List of sent requests - No need for mutex since only the peer thread will access this */
	
	/* Connexion information */
	
	/* Security information (callbacks) */
	
	/* Supported applications (result of CER/CEA) */
	size_t		 p_app_size;	/* The size of the array pointed by p_app_list */
	peer_appl_t	*p_app_list;	/* points to a 0-terminated array of supported applications */
	
	/* what else? */
	
} _peer_t;

	


/* Initialize the module */
int peer_init ( void )
{
	TRACE_ENTRY( "" );
	TRACE_DEBUG (INFO, "@@@ Not implemented yet." );
	return 0;
}

/* End of the module */
int peer_fini ( void )
{
	TRACE_ENTRY( "" );
	TRACE_DEBUG (INFO, "@@@ Not implemented yet." );
	return 0;
}

/* Add a new peer */
int peer_add ( peer_add_t * data )
{
	TRACE_ENTRY( "%p", data );
	TRACE_DEBUG (INFO, "@@@ Not implemented yet." );
	return 0;
}

/* Allow the peers to start connexions */
int peer_start ( void )
{
	TRACE_ENTRY( "" );
	TRACE_DEBUG (INFO, "@@@ %s: not implemented yet.", __FUNCTION__ );
	return ENOTSUP;
}

/* Get some information from a peer_t opaque object */
int peer_get ( peer_t * peer, peer_info_t what, void **result )
{
	TRACE_ENTRY( "%p %d %p", peer, what, result );
	TRACE_DEBUG (INFO, "@@@ %s: not implemented yet.", __FUNCTION__ );
	return ENOTSUP;
}

/* Queue a message in the given peer */
int peer_send ( peer_t * peer, msg_t * msg )
{
	TRACE_ENTRY( "%p %p", peer, msg );
	TRACE_DEBUG (INFO, "@@@ %s: not implemented yet.", __FUNCTION__ );
	return ENOTSUP;
}
