/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* Internal waaad headers
 * 
 *  This file simply includes all the headers of the waaad daemon in 
 * order to simplify the #include process in the C files.
 */
#ifndef _WAAAD_INTERNAL_H
#define _WAAAD_INTERNAL_H

/* Configuration generated by the configure script. Contains host-specific environment information */
#include "config.h"

/* The logging facility. 
All daemon messages are sent to this module after initialization.
Extensions may also choose to send their messages to this module, or handle their own logging mechanism.
log_* functions.
*/
#include "log.h"

/* The dictionary module. 
Provides description of all Diameter commands and AVP, and also vendors, applications, and rules. 
dict_* functions.
*/
#include "dictionary.h"

/* The extensions module. 
Provides support for loading the extensions to the waaad daemon.
ext_* functions.
*/
#include "extensions.h"

/* The configuration module.
Provides support for parsing the configuration file of the daemon. 
It uses the conf-gram.y and conf-token.l parsers.
conf_* and yy* functions.
*/
#include "conf.h"

/* The peer management module.
Provides management of and basic control over the peers to which the local host is connected.
peer_* functions.
*/
#include "peer.h"

/* The message module.
Provides functions to manipulate Diameter messages structures.
msg_* functions.
*/
#include "message.h"

/* The message queues module.
Provides functions to queue and retrieve messages in FIFO order, to pass messages between threads.
meq_* functions.
*/
#include "queues.h"

/* The routing module.
Provides functions and threads to route the messages between the global queues and peers queues.
rt_* functions.
*/
#include "routing.h"

/* The session module.
Provides basical support to implement stateful session management.
sess_* functions.
*/
#include "session.h"

/* The dispatch module.
Provides functions and threads to handle locally the Diameter messages in extensions.
disp_* functions.
*/
#include "dispatch.h"

/* The security module.
Provides the support for protecting the Diameter messages that are exchanges on the network.
The actual protection (cipher, ...) is provided by extensions.
sec_* functions.
*/
#include "security.h"


#endif /* _WAAAD_INTERNAL_H */
